/**
 *
 * 基础系统相关接口
 *
 */
import request from '@/utils/request';
import qs from 'qs';

export const getMenuQuery = (params) => {
	return request({
		url: '/nh-farm/cp/system/menuQuery',
		method: 'get',
		params
	});
};

// 获取菜单列表
export const getMenuList = (params) => {
	return request({
		url: '/nh-farm/cp/system/findSysAndMenus',
		method: 'get',
		params
	});
};

// 数据同步
export const dataSysSync = (data) => {
	return request({
		url: '/nh-farm/systemSetting/sysSync',
		method: 'post',
		data
	});
};
// 查询结构
export const dataSysList = (data) => {
	return request({
		url: '/nh-farm/systemSetting/sysList',
		method: 'post',
		data
	});
};
// 修改顺序
export const editSysIndex = (data) => {
	return request({
		url: '/nh-farm/systemSetting/editSys',
		method: 'post',
		data
	});
};

export function getAdminList(params) {
	return request({
		url: '/nh-farm/cp/admin/list',
		method: 'GET',
		params
	});
}
// 初始化密码
export function initPassword(data) {
	return request({
		url: '/nh-farm/cp/admin/editPass',
		method: 'POST',
		data
	});
}

// 机构数据
export const bankTree = (params) => {
	return request({
		url: '/nh-farm/cp/systemManagement/area/bankTree',
		method: 'get',
		params
	});
};

export function addMenuOrButton(menu) {
	return request({
		url: '/nh-farm/cp/system/addMenus',
		method: 'POST',
		data: qs.stringify(menu),
		headers: {
			'content-type': 'application/x-www-form-urlencoded; charset=UTF-8'
		}
	});
}

export function addSys(farm) {
	return request({
		url: '/nh-farm/cp/system/add',
		method: 'POST',
		data: qs.stringify(farm),
		headers: {
			'content-type': 'application/x-www-form-urlencoded; charset=UTF-8'
		}
	});
}

// '
export function getMenuById(KID) {
	return request({
		url: '/nh-farm/cp/system/edit_form',
		method: 'GET',
		params: { kid: KID }
	});
}

export function editSys(sysFormTemp) {
	return request({
		url: '/nh-farm/cp/system/edit',
		method: 'POST',
		params: sysFormTemp
	});
}
export function getRoleAuthList(params) {
	return request({
		url: '/nh-farm/cp/system/findSysAndMenusByAuths',
		method: 'GET',
		params
	});
}

export function validateMenuCodeExist(oldCode, newCode) {
	return request({
		url: '/nh-farm/cp/system/menusCodeExists',
		method: 'POST',
		params: { oldSysCode: oldCode, newMenuCode: newCode }
	});
}

export function validateSysCodeExist(oldCode) {
	return request({
		url: '/nh-farm/cp/system/sysCodeExists',
		method: 'POST',
		params: { sysCode: oldCode }
	});
}

export function deleteMenu(KID) {
	return request({
		url: '/nh-farm/cp/system/delete',
		method: 'GET',
		params: { kid: KID }
	});
}

/**
 * 获取用户当前机构
 */
export function getUserBankList() {
	return request({
		url: '/nh-farm/cp/systemManagement/area/bank_list',
		method: 'GET'
	});
}

/**  ============================= 机构相关接口============================= */

/**
 * 获取机构列表
 * @param {*} search
 */
export function getBankList(search) {
	return request({
		url: '/nh-farm/cp/systemManagement/area/bank_list',
		method: 'GET',
		params: search
	});
}

/**
 * 新增机构
 * @param {*} bank
 */
export function addBank(bank) {
	return request({
		url: '/nh-farm/cp/systemManagement/area/add',
		method: 'POST',
		data: qs.stringify(bank),
		headers: { 'content-type': 'application/x-www-form-urlencoded' }
	});
}

/**
 * 删除机构
 * @param {*} kid
 */
export function deleteBank(KID) {
	return request({
		url: '/nh-farm/cp/systemManagement/area/delete',
		method: 'GET',
		params: { kid: KID }
	});
}

/**
 * 根据id查询机构信息
 * @param {*} kid
 */
export function findBankById(KID) {
	return request({
		url: '/nh-farm/cp/systemManagement/area/update_form',
		method: 'GET',
		params: { kid: KID }
	});
}

/**
 * 编辑机构
 * @param {*} bank
 */
export function editBank(bank) {
	return request({
		url: '/nh-farm/cp/systemManagement/area/update',
		method: 'POST',
		data: qs.stringify(bank),
		headers: { 'content-type': 'application/x-www-form-urlencoded' }
	});
}

/**
 * 验证机构编码
 * @param {*} code
 */
export function validateBankCode(newCode, oldCode) {
	return request({
		url: '/nh-farm/cp/systemManagement/area/listFindByCode',
		method: 'GET',
		params: { oldBankFatherCode: oldCode, newBankCode: newCode }
	});
}

/**
 * 获取角色权限列表
 * @param {*} search
 */

/**
 * 新增角色
 * @param {*} role
 */
export function addRole(role) {
	return request({
		url: '/nh-farm/cp/systemManagement/role/add',
		method: 'POST',
		data: qs.stringify(role),
		headers: {
			'content-type': 'application/x-www-form-urlencoded; charset=UTF-8'
		}
	});
}

/**
 * 根据id删除角色
 * @param {*} kid
 */
export function deleteRole(kid) {
	return request({
		url: '/nh-farm/cp/systemManagement/role/delete',
		method: 'GET',
		params: { kid: kid }
	});
}

/**
 * 根据id获取角色权限
 * @param {*} kid
 */
export function getRoleById(KID) {
	return request({
		url: '/nh-farm/cp/systemManagement/role/update_form ',
		method: 'POST',
		params: { kid: KID }
	});
}

/**
 * 编辑角色
 * @param {*} roleAuth
 */
export function editRole(role) {
	return request({
		url: '/nh-farm/cp/systemManagement/role/update',
		method: 'POST',
		data: qs.stringify(role),
		headers: {
			'content-type': 'application/x-www-form-urlencoded; charset=UTF-8'
		}
	});
}

/**
 * 校验角色编码唯一性
 * @param {*} roleCode
 */
export function validateRoleCodeExist(roleCode, areaCode) {
	return request({
		url: '/nh-farm/cp/systemManagement/role/roleIdExists',
		method: 'POST',
		params: { roleId: roleCode, roleCode: areaCode }
	});
}
/**  ============================= 系统菜单相关接口============================= */

/**
 * 获取角色
 * @param {*} bankId
 */
export function getUserRoleList(params) {
	return request({
		url: '/nh-farm/cp/systemManagement/role/list',
		method: 'GET',
		params
	});
}
// 获取机构

/**
 * 新增用户
 * @param {*} user
 */
export function adduser(data) {
	return request({
		url: '/nh-farm/cp/admin/add',
		method: 'POST',
		data
		// : qs.stringify(user),
		// headers: {
		//   'content-type': 'application/x-www-form-urlencoded; charset=UTF-8'
		// }
	});
}

// 查询用户是否存在
export function getUserExist(kid) {
	return request({
		url: '/nh-farm/cp/admin/userExists',
		method: 'POST',
		data: { userId: kid }
	});
}

/**
 * 根据id删除用户
 * @param {*} kid
 */
export function deleteUser(kid) {
	return request({
		url: '/nh-farm/cp/admin/delete',
		method: 'GET',
		params: { kid: kid }
	});
}

/**  ============================= 员工信息维护相关接口============================= */
/**
 * 获取员工信息列表
 * @param {*} search
 */
export function getInfoList(search) {
	return request({
		url: '/nh-farm/cp/admin/list',
		method: 'GET',
		params: search
	});
}

/**
 * 修改员工信息
 * @param {*} search
 */

export function editInfo(data) {
	return request({
		url: '/nh-farm/cp/admin/edit',
		method: 'POST',
		data
		// qs.stringify(info),
		// headers: {
		//   'content-type': 'application/x-www-form-urlencoded; charset=UTF-8'
		// }
	});
}

/**  ============================= 数据字典相关接口============================= */
/**
 * 获取数据字典列表
 * @param {*} search
 */
export function getParamList(search) {
	return request({
		url: '/nh-bank/cp/param/paramsys',
		method: 'GET',
		params: search
	});
}

/**
 * 验证数据编码是否存在
 * @param {*} search
 */
export function validateCode(paramCode) {
	return request({
		url: '/nh-bank/cp/param/paramsCodeExists',
		method: 'POST',
		params: { paramCode: paramCode }
	});
}

/**
 * 验证数据子编码是否存在
 * @param {*} search
 */
export function validateChildCode(fatherParamCode, newBankCode) {
	return request({
		url: '/nh-bank/cp/param/paramsCodeExists',
		method: 'POST',
		params: { fatherParamCode: fatherParamCode, newBankCode: newBankCode }
	});
}

/**
 * 添加数据字典
 * @param {*} search
 */
export function addParam(param) {
	return request({
		url: '/nh-bank/cp/param/addParamLx',
		method: 'POST',
		data: qs.stringify(param),
		headers: {
			'content-type': 'application/x-www-form-urlencoded; charset=UTF-8'
		}
	});
}

/**
 * 删除数据字典
 * @param {*} noticeId
 */
export function deleteParam(kid) {
	return request({
		url: '/nh-bank/cp/param/delete',
		method: 'GET',
		params: { kid: kid }
	});
}

/**
 * 修改数据字典
 * @param {*} search
 */
export function updateParam(param) {
	return request({
		url: '/nh-bank/cp/param/edit',
		method: 'POST',
		data: qs.stringify(param),
		headers: {
			'content-type': 'application/x-www-form-urlencoded; charset=UTF-8'
		}
	});
}

/**
 * 获取数据字典信息
 * @param {*} noticeId
 */
export function findParamById(kid) {
	return request({
		url: '/nh-bank/cp/param/edit_form',
		method: 'GET',
		params: { kid: kid }
	});
}

/**
 * 添加子数据字典
 * @param {*} search
 */
export function addChildParam(param) {
	return request({
		url: '/nh-bank/cp/param/add',
		method: 'POST',
		data: qs.stringify(param),
		headers: {
			'content-type': 'application/x-www-form-urlencoded; charset=UTF-8'
		}
	});
}

// 查询角色是否存在
export function getroleExist(roleId, areaCode) {
	return request({
		url: '/nh-farm/cp/systemManagement/role/roleIdExists',
		method: 'post',
		params: { roleId: roleId, areaCode: areaCode }
	});
}

// ----------------------------------------系统设置----------------------------
export function getEditData(data) {
	return request({
		url: '/nh-farm/cp/system/edit_data',
		method: 'post',
		data
	});
}

// 解锁接口
export function unlock(password) {
	return request({
		url: '/nh-farm/debLocking',
		method: 'POST',
		data: qs.stringify({ password }),
		headers: { 'content-type': 'application/x-www-form-urlencoded' }
	});
}

//
export const findFarmAuth = (data) => {
	return request({
		url: '/nh-farm-finance/farmAuth/findFarmAuth',
		method: 'POST',
		data
	});
};

export const findFarmRole = (params) => {
	return request({
		url: '/nh-farm-finance/farmAuth/findFarmRole',
		method: 'get',
		params
	});
};

// 财务人员权限维护
// 保存
export const addFarmAuth = (data) => {
	return request({
		url: '/nh-farm-finance/farmAuth/addFarmAuth',
		method: 'post',
		data
	});
};
// 删除
export const deleteFarmAuth = (data) => {
	return request({
		url: '/nh-farm-finance/farmAuth/deleteFarmAuth',
		method: 'post',
		data
	});
};

//
export const getBankByPhone = (data) => {
	return request({
		url: '/nh-farm-finance/phoneBankRole/getBankByPhone',
		method: 'post',
		data
	});
};
export const updateBankByPhone = (data) => {
	return request({
		url: '/nh-farm-finance/phoneBankRole/updateBankByPhone',
		method: 'post',
		data
	});
};
export const getRoleByKid = (data) => {
	return request({
		url: '/nh-farm-finance/phoneBankRole/getRoleByKid',
		method: 'post',
		data
	});
};
export const updateRoleById = (data) => {
	return request({
		url: '/nh-farm-finance/phoneBankRole/updateRoleById',
		method: 'post',
		data
	});
};
export const caiwuRoleList = (params) => {
	return request({
		url: '/nh-farm-finance/phoneBankRole/caiwuRoleList',
		method: 'get',
		params
	});
};
export const caiwuRoleListLower = (data) => {
	return request({
		url: '/nh-farm-finance/phoneBankRole/caiwuRoleListLower',
		method: 'post',
		data
	});
};
