import request from '@/utils/request';
import { parseStrEmpty } from '@/utils/ruoyi';

// 查询用户列表
export function listUser(query) {
	return request({
		url: '/system/users',
		method: 'get',
		params: query
	});
}

// 查询用户详细
export function getUser(userId) {
	return request({
		url: '/system/user/' + parseStrEmpty(userId),
		method: 'get'
	});
}

// 新增用户
export function addUser(data) {
	return request({
		url: '/system/user',
		method: 'post',
		data: data
	});
}

// 修改用户
export function updateUser(data) {
	return request({
		url: '/system/users/' + data.kid,
		method: 'put',
		data: data
	});
}

// 删除用户
export function delUser(userId) {
	return request({
		url: '/system/user/' + userId,
		method: 'delete'
	});
}

// 用户密码重置
export function resetUserPwd(userId, password) {
	const data = {
		userId,
		password
	};
	return request({
		url: '/system/user/resetPwd',
		method: 'put',
		data: data
	});
}

// 用户状态修改
export function changeUserStatus(kid, status) {
	return request({
		url: `/system/users/${kid}/${status}`,
		method: 'put'
	});
}

// 查询用户个人信息
export function getUserProfile() {
	return request({
		url: '/system/user/profile',
		method: 'get'
	});
}

// 修改用户个人信息
export function updateUserProfile(data) {
	return request({
		url: '/system/user/profile',
		method: 'put',
		data: data
	});
}

// 用户密码重置
export function updateUserPwd(data) {
	return request({
		url: '/system/user/passwd/' + data.kid,
		method: 'put',
		params: data
	});
}

// 用户头像上传
export function uploadAvatar(data) {
	return request({
		baseURL: '/nh-cloud',
		url: '/api/file/upload_new',
		method: 'post',
		data: data
	});
}

// 查询授权角色
export function getAuthRole(userId) {
	return request({
		url: '/system/user/authRole/' + userId,
		method: 'get'
	});
}

// 保存授权角色
export function updateAuthRole(data) {
	return request({
		url: '/system/user/authRole',
		method: 'put',
		params: data
	});
}

export function addUserRoleBank(data) {
	return request({
		url: '/system/userRoleBank',
		method: 'post',
		data
	});
}
export function userRoleBankData(params) {
	return request({
		url: `/system/userRoleBank/${params.userKid}/${params.menuKid}`,
		method: 'get'
	});
}

export function updateUserRoleBank(data) {
	return request({
		url: `/system/userRoleBank/${data.userKid}/${data.menuKid}`,
		method: 'put',
		data
	});
}
export function userLock(data) {
	return request({
		url: `/system/userLock/${data.username}`,
		method: 'delete',
		data
	});
}
export function localUser(data) {
	return request({
		baseURL: '/basic-gateway',
		url: `/system/localUser/${data.kid}`,
		method: 'put',
		data
	});
}
export function localUsers(params) {
	return request({
		baseURL: '/basic-gateway',
		url: `/system/localUser`,
		method: 'get',
		params
	});
}
