import { stringToByte, navPrive } from '@/utils/utils.js';
// import * as imageConversion from 'image-conversion';
// import { getFile } from '@/api/modules/office';
import { mapState } from 'vuex';
import { getToken } from '@/utils/auth';
export default {
	data() {
		return {
			replaceId: 0,
			updateToken: true,
			token: {},
			parentId: -1,
			parentData: [],
			parentProps: {
				value: 'storage_id',
				label: 'name',
				children: 'children',
				checkStrictly: true,
				emitPath: false
			},
			headerObj: {}
		};
	},
	computed: {
		...mapState({
			userInfo: (state) => state.user.userInfo,
			bankInfo: (state) => state.user.currSelectBank,
			accountBook: (state) => state.user.currAccountBook,
			accountMessage: (state) => state.user.currAccountMessage
		})
	},
	mounted() {
		this.headerObj = {
			Authorization: 'Bearer ' + getToken()
		};
		this.$set(this.params, 'path', `${this.userInfo.bankCode}/${this.$store.state.permission.currSystem.menuCode}`);
		this.$set(this.params, 'bankCode', this.bankInfo.bankCode);
		if (this.accountMessage) {
			this.$set(this.params, 'accountingPeriod', this.accountMessage.accountingPeriod);
			this.$set(this.params, 'bookId', this.accountMessage.bookId);
			this.$set(this.params, 'year', this.accountMessage.year);
		}
	},
	methods: {
		// 删除资源
		handleRemove(file, fileList) {
			if (file.status === 'success' && file.response) {
				const response = file.response.data;
				if (response && response[0].storage_id) {
					const storageId = response[0].storage_id;
					// eslint-disable-next-line no-undef
					delStorageList(Array.isArray(storageId) ? storageId : [storageId]);
				}
			}
			this.$emit('upload', fileList);
		},
		// 资源预览
		handlePreview(file) {
			if (file.status === 'success') {
				navPrive(file.url);
			}
		},
		// 上传文件之前的钩子
		handleBeforeUpload(file) {
			return new Promise((resolve, reject) => {
				const fielMaxSize = stringToByte(this.fileSize);
				if (fielMaxSize > 0 && file.size > fielMaxSize) {
					this.$message.error(`上传资源大小不能超过 ${this.fileSize}`);
					// eslint-disable-next-line prefer-promise-reject-errors
					return reject(false);
				}

				const suffix = file.name.toLowerCase().split('.').splice(-1).toString();
				const allImageType =
					'.svgz,.pjp,.jfj,.ico,.tiff,.avif,.svg,.jpg,.tif,.gif,.png,.xbm,.pjpeg,.bmp,.wbp';
				const checkSuffix = this.accept === 'image/*' ? allImageType : this.accept;
				const noSuffix = ['*/*', '*'];
				if (!noSuffix.includes(checkSuffix) && checkSuffix.indexOf(suffix) === -1) {
					this.$message.error('上传资源的文件后缀不允许上传');
					// eslint-disable-next-line prefer-promise-reject-errors
					return reject(false);
				}
				// 自动上传时,"确定"按钮将改变状态
				if (this.autoUpload) {
					this.loading = true;
				}
				// const imgType = file.type;
				// const imgName = file.name;
				// let compress = fielMaxSize; // 假设图片限制不能大于2M
				// let sizeOver = file.size / 1024 > compress; //文件大小 是否大于指定大小
				// if (sizeOver && compress) {
				// 	//大于2M进行压缩
				// 	imageConversion
				// 		.compressAccurately(file, {
				// 			size: compress, //需要压缩的大小
				// 			accuracy: 0.9 //精度 0.8-0.99之间 默认值0.95
				// 			// type: 'image/jpeg',
				// 			// width: 105,
				// 			// height: 105,
				// 			// orientation: 2,
				// 			// scale: 0.5
				// 		})
				// 		.then((res) => {
				// 			file = res; //把得到的新的图片文件赋值给原文件,然后进行上传   得到的是bold对象
				// 			file = new window.File([file], imgName, { type: imgType }); //将bold对象转换为file文件
				// 			resolve(true); // 通过resolve将Promise函数返回成功回调,进行后面操作
				// 		})
				// 		.catch((err) => {
				// 			resolve(true); // 通过resolve将Promise函数返回成功回调,进行后面操作
				// 		});
				// }
				resolve(true); // 通过resolve将Promise函数返回成功回调,进行后面操作
			});
		},
		// 文件上传成功时的钩子
		handleSuccess(response, file, fileList) {
			this.loading = false;
			if (response.code === 200) {
				this.$emit('upload', fileList);
				// this.$emit('confirm', fileList)
				this.handleChange(fileList);
				this.$notify({
					title: '提示',
					message: '附件上传成功',
					type: 'success'
				});
				return;
			}

			this.handleError(response.message || response.result, file, fileList);
		},
		// 文件上传失败时的钩子
		handleError(_err, file, fileList) {
			this.loading = false;
			this.$notify({
				title: '提示',
				message: `资源：${file.name} 上传失败 ${_err}`,
				type: 'warning'
			});

			for (let i = fileList.length - 1; i >= 0; i--) {
				if (file === fileList[i]) {
					fileList.splice(i, 1);
					this.$emit('upload', fileList);
					break;
				}
			}

			this.handleChange(fileList);
		},
		// 文件超出个数限制时的钩子
		handleExceed(files, fileList) {
			if (fileList.length >= this.limit) {
				this.$message.warning(`最多只能上传 ${this.limit} 个文件`);
				return;
			}

			if (files.length + fileList.length > this.limit) {
				const count = this.limit - fileList.length;
				this.$message.warning(`上传数量超出限制，最多还能选择 ${count} 个文件`);
			}
		},
		// "确定"按钮状态转换
		handleChange(fileList) {
			if (this.autoUpload) {
				if (Object.keys(fileList).every((item) => fileList[item].status === 'success')) {
					this.loading = false;
				}
			}
		}
	}
};
