import Vue from 'vue';
import store from '@/store';
// eslint-disable-next-line import/no-duplicates
import Router from 'vue-router';
// eslint-disable-next-line import/no-duplicates
import VueRouter from 'vue-router';
import { getToken } from '@/utils/auth';
// import actiConstantRouter from './modules/fm';
import { fixHeaderSum, fixFotterSum } from '@/utils/fix_table.js';

import { routers } from './routers';
import axios from 'axios';
import '@/utils/filter';
/* Layout */
import Layout from '@/layout';
import NProgress from 'nprogress';
import 'nprogress/nprogress.css';
// eslint-disable-next-line no-unused-vars
import { MessageBox, Message } from 'element-ui';
import { isEmpty } from 'lodash';
NProgress.configure({ showSpinner: false });

// fix vue-router NavigationDuplicated
const VueRouterPush = VueRouter.prototype.push;
VueRouter.prototype.push = function push(location) {
    return VueRouterPush.call(this, location).catch((err) => err);
};
const VueRouterReplace = VueRouter.prototype.replace;
VueRouter.prototype.replace = function replace(location) {
    return VueRouterReplace.call(this, location).catch((err) => err);
};
Vue.use(VueRouter);
// 公共路由
export const constantRoutes = [{
        path: '/',
        redirect: `/nh-sale/home`,
        hidden: true
    },
    ...routers,
    {
        path: '/redirect',
        component: Layout,
        hidden: true,
        children: [{
            path: '/redirect/:path(.*)',
            component: () =>
                import ('@/views/system/redirect')
        }]
    },
    {
        path: '/401',
        component: () =>
            import ('@/views/system/error/401.vue'),
        hidden: true
    },
    {
        path: '*',
        component: () =>
            import ('@/views/system/error/404'),
        hidden: true
    },
    {
        path: '*',
        component: () =>
            import ('@/views/system/jump'),
        hidden: true
    }
];

// 动态路由，基于用户权限动态去加载
export const dynamicRoutes = [];

const router = new VueRouter({
    // mode: 'history', // 去掉url中的#
    scrollBehavior: () => ({ y: 0 }),
    routes: constantRoutes
});
// 防止连续点击多次路由报错
const routerPush = Router.prototype.push;
Router.prototype.push = function push(location) {
    return routerPush.call(this, location).catch((err) => err);
};

const whiteList = ['/401', '/404', '/redirect', '/nh-sale/home'];

router.beforeEach(async(to, from, next) => {
    NProgress.start();
    if (getToken()) {
        to.meta.title && store.dispatch('settings/setTitle', to.meta.title);
        const path = to.path;
        const menuCode = path.split('/')[1];
        // 多系统
        const systems = ['nh-sale'];
        const allSystems = store.state.permission.system;
        if (systems.includes(menuCode)) {
            for (let index = 0; index < allSystems.length; index++) {
                const e = allSystems[index];
                if (e.menuCode === menuCode) {
                    store.commit('setCurrSystem', e);
                    break;
                }
            }
        }
        // console.log(store.state.permission.sidebarRouters)
        if (isEmpty(store.state.permission.sidebarRouters)) {
            await store.dispatch('getMenu');
        }

        if (path === '/login') {
            next({ path: '/' });
            NProgress.done();
        } else {
            const isNext = to.path === '/401' || Vue.prototype.$permission(path);
            if (isNext) {
                next();
                NProgress.done();
            } else {
                next({ path: '/401' });
                NProgress.done();
            }
        }
    } else {
        if (whiteList.indexOf(to.path) !== -1) {
            next();
        } else {
            NProgress.done();
            window.location.href = '/nh-desktop/#/login';
        }
    }
});

router.afterEach(async(to, from, next) => {
    NProgress.done();

    // changeTitle(`${store.state.user.systemTitle.text}`);

    fixHeaderSum();
    fixFotterSum();
});

export default router;
