const state = {
	footerVisitedViews: [],
	footerCachedViews: []
};

const mutations = {
	addFooterVisitedView(state, view) {
		if (state.footerVisitedViews.some((v) => v.path === view.path)) return;
		state.footerVisitedViews.push(
			Object.assign({}, view, {
				title: view.systemName || 'no-name'
			})
		);
	},
	addFooterCachedView(state, view) {
		if (state.footerCachedViews.includes(view.systemName)) return;
		if (view.meta && !view.meta.noCache) {
			state.footerCachedViews.push(view.systemName);
		}
	},
	delVisitedView(state, view) {
		for (const [i, v] of state.footerVisitedViews.entries()) {
			if (v.path === view.path) {
				state.footerVisitedViews.splice(i, 1);
				break;
			}
		}
	},
	delCachedView(state, view) {
		const index = state.footerCachedViews.indexOf(view.systemName);
		index > -1 && state.footerCachedViews.splice(index, 1);
	},
	delOthersVisitedViews(state, view) {
		state.footerVisitedViews = state.footerVisitedViews.filter((v) => {
			return v.meta.affix || v.path === view.path;
		});
	},
	DEL_OTHERS_CACHED_VIEWS: (state, view) => {
		const index = state.footerCachedViews.indexOf(view.systemName);
		if (index > -1) {
			state.footerCachedViews = state.footerCachedViews.slice(index, index + 1);
		} else {
			state.footerCachedViews = [];
		}
	},

	DEL_ALL_VISITED_VIEWS: (state) => {
		// keep affix tags
		const affixTags = state.footerVisitedViews.filter((tag) => tag.meta.affix);
		state.footerVisitedViews = affixTags;
	},
	DEL_ALL_CACHED_VIEWS: (state) => {
		state.footerCachedViews = [];
	},

	UPDATE_VISITED_VIEW: (state, view) => {
		for (let v of state.footerVisitedViews) {
			if (v.path === view.path) {
				v = Object.assign(v, view);
				break;
			}
		}
	},

	DEL_RIGHT_VIEWS: (state, view) => {
		const index = state.footerVisitedViews.findIndex((v) => v.path === view.path);
		if (index === -1) {
			return;
		}
		state.footerVisitedViews = state.footerVisitedViews.filter((item, idx) => {
			if (idx <= index || (item.meta && item.meta.affix)) {
				return true;
			}
			const i = state.footerCachedViews.indexOf(item.name);
			if (i > -1) {
				state.footerCachedViews.splice(i, 1);
			}
			return false;
		});
	},

	DEL_LEFT_VIEWS: (state, view) => {
		const index = state.footerVisitedViews.findIndex((v) => v.path === view.path);
		if (index === -1) {
			return;
		}
		state.footerVisitedViews = state.footerVisitedViews.filter((item, idx) => {
			if (idx >= index || (item.meta && item.meta.affix)) {
				return true;
			}
			const i = state.footerCachedViews.indexOf(item.name);
			if (i > -1) {
				state.footerCachedViews.splice(i, 1);
			}
			return false;
		});
	}
};

const actions = {
	addView({ dispatch }, view) {
		dispatch('addVisitedView', view);
		dispatch('addCachedView', view);
	},
	addVisitedView({ commit }, view) {
		commit('ADD_VISITED_VIEW', view);
	},
	addCachedView({ commit }, view) {
		commit('ADD_CACHED_VIEW', view);
	},

	delView({ dispatch, state }, view) {
		return new Promise((resolve) => {
			dispatch('delVisitedView', view);
			dispatch('delCachedView', view);
			const data = {
				footerVisitedViews: [...state.footerVisitedViews],
				footerCachedViews: [...state.footerCachedViews]
			};
			resolve(data);
		});
	},
	delVisitedView({ commit, state }, view) {
		return new Promise((resolve) => {
			commit('delVisitedView', view);
			resolve([...state.footerVisitedViews]);
		});
	},
	delCachedView({ commit, state }, view) {
		return new Promise((resolve) => {
			commit('delCachedView', view);
			resolve([...state.footerCachedViews]);
		});
	},

	delOthersViews({ dispatch, state }, view) {
		return new Promise((resolve) => {
			dispatch('delOthersVisitedViews', view);
			dispatch('delOthersCachedViews', view);
			resolve({
				footerVisitedViews: [...state.footerVisitedViews],
				footerCachedViews: [...state.footerCachedViews]
			});
		});
	},
	delOthersVisitedViews({ commit, state }, view) {
		return new Promise((resolve) => {
			commit('DEL_OTHERS_VISITED_VIEWS', view);
			resolve([...state.footerVisitedViews]);
		});
	},
	delOthersCachedViews({ commit, state }, view) {
		return new Promise((resolve) => {
			commit('DEL_OTHERS_CACHED_VIEWS', view);
			resolve([...state.footerCachedViews]);
		});
	},

	delAllViews({ dispatch, state }, view) {
		return new Promise((resolve) => {
			dispatch('delAllVisitedViews', view);
			dispatch('delAllCachedViews', view);
			resolve({
				footerVisitedViews: [...state.footerVisitedViews],
				footerCachedViews: [...state.footerCachedViews]
			});
		});
	},
	delAllVisitedViews({ commit, state }) {
		return new Promise((resolve) => {
			commit('DEL_ALL_VISITED_VIEWS');
			resolve([...state.footerVisitedViews]);
		});
	},
	delAllCachedViews({ commit, state }) {
		return new Promise((resolve) => {
			commit('DEL_ALL_CACHED_VIEWS');
			resolve([...state.footerCachedViews]);
		});
	},

	updateVisitedView({ commit }, view) {
		commit('UPDATE_VISITED_VIEW', view);
	},

	delRightTags({ commit }, view) {
		return new Promise((resolve) => {
			commit('DEL_RIGHT_VIEWS', view);
			resolve([...state.footerVisitedViews]);
		});
	},

	delLeftTags({ commit }, view) {
		return new Promise((resolve) => {
			commit('DEL_LEFT_VIEWS', view);
			resolve([...state.footerVisitedViews]);
		});
	},

	// eslint-disable-next-line no-unused-vars
	setFooterTagsView({ state }) {
		// store 赋值
		// state.footerVisitedViews = info;
		// 持久化
		window.localStorage.setItem('footerTagsView', JSON.stringify(state.footerVisitedViews));
	},
	loadFooterTagsView({ state }) {
		const footerTagsView = window.localStorage.getItem('footerTagsView');
		// store 赋值
		if (footerTagsView === null) {
			state.footerVisitedViews = [];
		} else {
			state.footerVisitedViews = JSON.parse(footerTagsView);
		}
	}
};

export default {
	namespaced: true,
	state,
	mutations,
	actions
};
