import auth from '@/plugins/auth';
import { constantRoutes } from '@/router';
import Layout from '@/layout/index';
import { sysQuery } from '@/api/system/login';
import { getMenus } from '@/api/index';
import { deepClone, arrToTree, createComparator, isEmpty } from '@/utils/utils';
import ParentView from '@/components/ParentView';
import InnerLink from '@/layout/components/InnerLink';


const whiteList = ['/401', '/404', '/redirect', `/nh-sale/home`, '/nh-sale/system/jump'];
const permission = {
    state: {
        routes: [],
        addRoutes: [],
        defaultRoutes: [],
        topbarRouters: [],
        sidebarRouters: [],

        cSidebarRouters: [],
        authKey: [],

        system: [],
        systemMenu: [],
        currSystem: null
    },
    mutations: {
        setCurrSystem: (state, obj) => {
            state.currSystem = obj;
        },
        SET_ROUTES: (state, routes) => {
            state.addRoutes = routes;
            state.routes = constantRoutes.concat(routes);
        },
        SET_DEFAULT_ROUTES: (state, routes) => {
            state.defaultRoutes = constantRoutes.concat(routes);
        },
        SET_TOPBAR_ROUTES: (state, routes) => {
            state.topbarRouters = routes;
        },
        SET_SIDEBAR_ROUTERS: (state, routes) => {
            state.sidebarRouters = routes;
        },

        setSystem: (state, system) => {
            state.system = system;
            // 持久化
            window.localStorage.setItem('system', JSON.stringify(system));
        },
        loadSystem: (state) => {
            const systems = window.localStorage.getItem('system');
            // store 赋值
            if (systems === null) {
                state.system = null;
            } else {
                state.system = JSON.parse(systems);
            }
        },

        // 设置系统菜单信息
        setSystemMenu(state, info) {
            const menuAffiliation = state.currSystem && state.currSystem.menuCode;
            // store 赋值
            state.sidebarRouters = info;
            // 持久化
            window.localStorage.setItem(menuAffiliation + '-systemMenu', JSON.stringify(info));
        },

        // 加载本地权限路由
        loadAuthKey: (state) => {
            const menuAffiliation = state.currSystem && state.currSystem.menuCode;
            const authKeyId = menuAffiliation + '-authKey';
            const authKey = window.localStorage.getItem(authKeyId);
            // store 赋值
            if (authKey === null) {
                let paths = [];
                whiteList.forEach((e) => {
                    paths.push(e);
                });
                paths = Array.from(new Set(paths));
                state.authKey = paths;
            } else {
                state.authKey = JSON.parse(authKey);
            }
        },
        // 设置权限路由
        setAuthKey(state, info) {
            const formatData = [];
            let paths = [];
            info.forEach((e) => {
                if (e.menuType === 'C' || e.menuType === 'F') {
                    formatData.push({
                        id: e.kid,
                        path: e.menuPath,
                        name: e.menuName,
                        ...e
                    });
                }
            });
            formatData.forEach((e) => {
                if (e.menuType === 'C') {
                    if (!isEmpty(e.path)) {
                        paths.push(e.path);
                    }
                }
                if (e.menuType === 'F') {
                    paths.push(e.menuPerm);
                }
            });
            whiteList.forEach((e) => {
                paths.push(e);
            });
            paths = Array.from(new Set(paths));

            state.authKey = paths;
            // // 持久化
            const menuAffiliation = state.currSystem && state.currSystem.menuCode;
            const authKeyId = menuAffiliation + '-authKey';
            window.localStorage.setItem(authKeyId, JSON.stringify(paths));
        }
    },
    actions: {
        // 生成路由
        // eslint-disable-next-line no-unused-vars
        GenerateRoutes({ commit }) {
            // eslint-disable-next-line no-unused-vars
            return new Promise((resolve) => {
                // 向后端请求路由数据
                // getRouters().then((res) => {
                // 	const sdata = JSON.parse(JSON.stringify(res.data));
                // 	const rdata = JSON.parse(JSON.stringify(res.data));
                // 	const sidebarRoutes = filterAsyncRouter(sdata);
                // 	const rewriteRoutes = filterAsyncRouter(rdata, false, true);
                // 	const asyncRoutes = filterDynamicRoutes(dynamicRoutes);
                // 	rewriteRoutes.push({ path: '*', redirect: '/404', hidden: true });
                // 	router.addRoutes(asyncRoutes);
                // 	commit('SET_ROUTES', rewriteRoutes);
                // 	commit('SET_SIDEBAR_ROUTERS', constantRoutes.concat(sidebarRoutes));
                // 	commit('SET_DEFAULT_ROUTES', sidebarRoutes);
                // 	commit('SET_TOPBAR_ROUTES', sidebarRoutes);
                // 	resolve(rewriteRoutes);
                // });
            });
        },

        // 获取系统菜单
        async getMenu({ state, dispatch, commit, rootState }) {
        	console.log(rootState)
        	if(rootState.user.currSelectBank){
	            const menuAffiliation = state.currSystem && state.currSystem.menuCode;
	            const res = await getMenus({ menuAffiliation, bankCode: rootState.user.currSelectBank.bankCode || '' });
	            const formatData = [];
	            res.result.forEach((e) => {
	                if (e.menuType === 'C' && e.showState === '1') {
	                    formatData.push({
	                        id: e.kid,
	                        path: e.menuPath,
	                        name: e.menuName,
	                        ...e
	                    });
	                }
	            });
	            const systemMenu = arrToTree(
	                deepClone(formatData),
	                'menuCode',
	                'upCode',
	                (item) => item.upCode === menuAffiliation
	            );
	            state.cSidebarRouters = res.result;
	            commit('setSystemMenu', systemMenu);
	
	            // await dispatch('getMenu');
	
	            commit('setAuthKey', res.result);
        	}else{
           		window.location.href = '/nh-desktop/#/login';
        	}

            // const treeToArr = (data, list = []) => {
            // 	data.map((e) => {
            // 		if (e.children) {
            // 			treeToArr(e.children, list);
            // 		}
            // 		list.push(e);
            // 		return list;
            // 	});
            // 	return list;
            // };
            // let systemMenuss = res.result.sort(createComparator('orderNum'));
            // systemMenuss = systemMenuss.filter((e) => {
            // 	return e.showState === '1';
            // });
            // // state.sidebarRouters = systemMenuss;
            // state.cSidebarRouters = treeToArr(systemMenuss);
        },

        // 读取菜单信息
        async loadSystemMenu({ state, dispatch }) {
            const menuAffiliation = state.currSystem && state.currSystem.menuCode;
            const storeId = menuAffiliation + "-systemMenu"
            const systemMenu = window.localStorage.getItem(storeId);
            // store 赋值
            if (systemMenu === null) {
                state.sidebarRouters = [];
                await dispatch('getMenu');
            } else {
                const treeToArr = (data, list = []) => {
                    data.map((e) => {
                        if (e.children) {
                            treeToArr(e.children, list);
                        }
                        list.push(e);
                        return list;
                    });
                    return list;
                };
                const systemMenus = JSON.parse(systemMenu);
                const systemMenuss = systemMenus.sort(createComparator('orderNum'));
                state.sidebarRouters = systemMenuss;
                state.cSidebarRouters = treeToArr(state.sidebarRouters);
            }
        },

        // 获取系统菜单权限
        async getSystem({ commit }) {
            const res = await sysQuery();
            const systems = res.result;
            const formatSystem = systems.map((e) => {
                return {
                    id: e.KID,
                    systemName: e.menuName,
                    systemCode: e.menuId,
                    icon: e.menuIcon,
                    path: e.menuPath,
                    ...e
                };
            });
            commit('setSystem', formatSystem);
            // const menuPermissions = menuList.map((e) => e.PATH).filter((s) => s);
            // setSystem(menuPermissions);
            // data.forEach((element) => {
            // 	element.kid = element.KID;
            // 	element.menu_code = element.SYS_CODE;
            // 	element.menu_name = element.SYS_NAME;
            // 	element.path = element.PATH;
            // 	element.icon = '@/assets/images/' + element.ICON;
            // 	systems.push(element);
            // 	this.menuList.push(element);
            // });

            // setSystem(systems, this);
        },
        // 系统跳转
        onSystemNav: async({ commit, dispatch }, system) => {
            commit('footerTagsView/addFooterVisitedView', system);
            const href = system.path;
            commit('tagsView/DEL_OTHERS_VISITED_VIEWSA');

            dispatch('tagsView/delAllViews');
            window.location.href = href;
            // 重载菜单
            dispatch('getMenu');
            // window.open(href);
        }
    }
};

// 遍历后台传来的路由字符串，转换为组件对象
// eslint-disable-next-line no-unused-vars
function filterAsyncRouter(asyncRouterMap, lastRouter = false, type = false) {
    return asyncRouterMap.filter((route) => {
        if (type && route.children) {
            route.children = filterChildren(route.children);
        }
        if (route.component) {
            // Layout ParentView 组件特殊处理
            if (route.component === 'Layout') {
                route.component = Layout;
            } else if (route.component === 'ParentView') {
                route.component = ParentView;
            } else if (route.component === 'InnerLink') {
                route.component = InnerLink;
            } else {
                route.component = loadView(route.component);
            }
        }
        if (route.children != null && route.children && route.children.length) {
            route.children = this.filterAsyncRouter(route.children, route, type);
        } else {
            delete route.children;
            delete route.redirect;
        }
        return true;
    });
}

function filterChildren(childrenMap, lastRouter = false) {
    var children = [];
    // eslint-disable-next-line no-unused-vars
    childrenMap.forEach((el, index) => {
        if (el.children && el.children.length) {
            if (el.component === 'ParentView' && !lastRouter) {
                el.children.forEach((c) => {
                    c.path = el.path + '/' + c.path;
                    if (c.children && c.children.length) {
                        children = children.concat(filterChildren(c.children, c));
                        return;
                    }
                    children.push(c);
                });
                return;
            }
        }
        if (lastRouter) {
            el.path = lastRouter.path + '/' + el.path;
        }
        children = children.concat(el);
    });
    return children;
}

// 动态路由遍历，验证是否具备权限
export function filterDynamicRoutes(routes) {
    const res = [];
    routes.forEach((route) => {
        if (route.permissions) {
            if (auth.hasPermiOr(route.permissions)) {
                res.push(route);
            }
        } else if (route.roles) {
            if (auth.hasRoleOr(route.roles)) {
                res.push(route);
            }
        }
    });
    return res;
}

export const loadView = (view) => {
    if (process.env.NODE_ENV === 'development') {
        return (resolve) => require([`@/views/${view}`], resolve);
    } else {
        // 使用 import 实现生产环境的路由懒加载
        return () =>
            import (`@/views/${view}`);
    }
};

export default permission;