import { login, bowlOut, logout, localUserInfo } from '@/api/system/login';
import { removeToken, getLoginToken, setToken, getToken } from '@/utils/auth';
import { isEmpty, changeTitle } from '@/utils/utils';
import { Message, MessageBox } from 'element-ui';
import { platformName, platformNamePlat } from '@/api/index';

const storePermissions = 'permissions';
const storeRoles = 'roles';

const user = {
    state: {
        loginToken: getLoginToken(),
        name: '',
        avatar: '',
        roles: [],
        permissions: [],

        // 修改密码弹框
        passwordVisbile: false,
        // 登录人信息
        userInfo: null,
        // 右上角机构
        currSelectBank: null,
        // {
        //   bankCode: "014000"
        //   bankCodeRule: "140000"
        //   bankLevel: 1
        //   bankName: "山西省"
        //   bankType: 9
        //   hasChildren: true
        //   upCode: "000"
        // }
        // 右上角账套
        currAccountBook: '',
        // 账套其他信息
        currAccountMessage: null,
        voucherState: true, // 凭证录入 是否为 编辑状态
        // {
        //   accountingPeriod: "2022-01"
        //   bookId: "402883c28281f1e1018281f32be30000"
        //   deInitialization: "true"
        //   initializeStatus: "false"
        //   monthBetween: Array(12)
        //   operableStatus: "false"
        //   yMonth: "2022-01"
        //   year: "2022"
        // }
        // 系统标题
        systemTitle: {
            text: '',
            eText: ''
        },
        systemTitlePlat: {},
        // 科目明细账查看凭证ids
        kmmxzIds: null,
        // 资产资源-出库
        zczyCkObj: null
    },

    mutations: {
        setKmmxzIds: (state, kmmxzIds) => {
            state.kmmxzIds = kmmxzIds;
        },
        setZczyCkObj: (state, zczyCkObj) => {
            state.zczyCkObj = zczyCkObj;
        },
        setLoginToken: (state, token) => {
            state.loginToken = token;
        },
        setName: (state, name) => {
            state.name = name;
        },
        setAvatar: (state, avatar) => {
            state.avatar = avatar;
        },
        setRoles: (state, roles) => {
            state.roles = roles;
            window.localStorage.setItem(storeRoles, JSON.stringify(roles));
        },
        loadRoles: (state) => {
            const roles = window.localStorage.getItem(storeRoles);
            // store 赋值
            if (roles === []) {
                state.roles = [];
            } else {
                state.roles = JSON.parse(roles);
            }
        },
        setPermissions: (state, permissions) => {
            state.permissions = permissions;
            window.localStorage.setItem(storePermissions, JSON.stringify(permissions));
        },
        loadPermissions: (state) => {
            const permissions = window.localStorage.getItem(storePermissions);
            // store 赋值
            if (permissions === []) {
                state.permissions = [];
            } else {
                state.permissions = JSON.parse(permissions);
            }
        },

        setPasswordVisbile: (state, passwordVisbile) => {
            state.passwordVisbile = passwordVisbile;
        },

        loadCurrSelectBank: (state) => {
            const currSelectBank = window.localStorage.getItem('selectBank');
            // store 赋值
            if (currSelectBank === null) {
                state.currSelectBank = null;
            } else {
                state.currSelectBank = JSON.parse(currSelectBank);
            }
        },

        loadCurrAccountBook: (state) => {
            const currAccountBook = window.localStorage.getItem('accountBook');
            // store 赋值
            if (currAccountBook === '') {
                state.currAccountBook = '';
            } else {
                state.currAccountBook = currAccountBook;
            }
        },

        setCurrAccountBook: (state, accountBook) => {
            state.currAccountBook = accountBook;
            window.localStorage.setItem('accountBook', accountBook);
        },

        loadCurrAccountMessage: (state) => {
            const currAccountMessage = window.localStorage.getItem('accountMessage');
            // store 赋值
            if (currAccountMessage === null) {
                state.currAccountMessage = null;
            } else {
                state.currAccountMessage = JSON.parse(currAccountMessage);
            }
        },
        setCurrAccountMessage: (state, currAccountMessage) => {
            state.currAccountMessage = currAccountMessage;
            window.localStorage.setItem('accountMessage', JSON.stringify(currAccountMessage));
        },
        loadSystemTitle: (state) => {
            const systemTitle = window.localStorage.getItem('systemTitle');
            // store 赋值
            if (isEmpty(systemTitle)) {
                state.systemTitle = {
                    text: '',
                    eText: '',
                    loginText: '',
                    driveText: ''
                };
            } else {
                state.systemTitle = JSON.parse(systemTitle);
            }
        },
        setSystemTitle: (state, systemTitle) => {
            state.systemTitle = systemTitle;
            window.localStorage.setItem('systemTitle', JSON.stringify(systemTitle));
        },

        setCurrSelectBankCom: (state, bankInfo) => {
            state.currSelectBank = bankInfo;
            window.localStorage.setItem('selectBank', JSON.stringify(bankInfo));
        },
        setVoucherState: (state, states) => {
            state.voucherState = states;
            // window.localStorage.setItem('selectBank', JSON.stringify(bankInfo));
        },
        loadSystemTitlPlat: (state) => {
            const systemTitlePlat = window.localStorage.getItem('systemTitlePlat');
            // store 赋值
            if (isEmpty(systemTitlePlat)) {
                state.systemTitlePlat = {
                    text: '',
                    eText: '',
                    loginText: '',
                    driveText: '',
                    remark: ''
                };
            } else {
                state.systemTitlePlat = JSON.parse(systemTitlePlat);
            }
        },
        setSystemTitlePlat: (state, systemTitlePlat) => {
            state.systemTitlePlat = systemTitlePlat;
            if (systemTitlePlat.platformNameLogin) {
                var currentIcon = systemTitlePlat.imagesUrl;
                var link = document.querySelector('link[rel*="icon"]');
                link.href = currentIcon;
                changeTitle(`${systemTitlePlat.platformNameLogin}`);
            }
            window.localStorage.setItem('systemTitlePlat', JSON.stringify(systemTitlePlat));
        }
    },

    actions: {
        async getPlatformNamePlat({ commit }) {
            try {
                const res = await platformNamePlat();
                const info = res.result;
                commit('setSystemTitlePlat', info);
            } catch (error) {
                console.log(error);
            }
        },
        setCurrSelectBank: async({ commit }, bankInfo) => {
            console.log(bankInfo)
            commit('setCurrSelectBankCom', bankInfo);
        },
        // 获取标题
        async getPlatformName({ commit }) {
            try {
                const res = await platformName();
                const info = res.result;
                commit('setSystemTitle', {
                    text: info.platformNameChinese,
                    eText: info.platformNameEnglish,
                    loginText: info.platformNameLogin,
                    driveText: info.platformNameDrive
                });
            } catch (error) {
                console.log(error);
            }
        },
        // 登录
        // eslint-disable-next-line no-unused-vars
        async Login({ commit, state, dispatch }, userInfo) {
            try {
                const loginRes = await login(userInfo);
                setToken(loginRes.result);
                if (loginRes.code !== 200) {
                    // Message.closeAll();
                    if (loginRes.code === 2000503) {
                        MessageBox.confirm('该账号已在其他客户端登录，是否强制退出?', '提示', {
                                confirmButtonText: '确定',
                                cancelButtonText: '取消',
                                type: 'warning'
                            })
                            .then(async() => {
                                const token = getToken();

                                removeToken();
                                window.localStorage.clear();
                                const logoutRes = await bowlOut({ token });
                                if (logoutRes.code !== 200) {
                                    Message.warning(logoutRes.message);
                                }
                            })
                            .catch(() => {});
                    }
                } else {
                    await dispatch('GetInfo');
                }
                return loginRes;
                // eslint-disable-next-line no-empty
            } catch (error) {}
        },

        // 获取用户信息
        async GetInfo({ commit, dispatch }) {
            try {
                const res = await localUserInfo();
                const user = res.result;
                dispatch('setUserInfo', user);
                commit('setName', user.userName);
                commit('setAvatar', user.avatar);
                return res;
            } catch (error) {
                return error;
            }
        },

        Logout({ commit }, { confirm = false } = {}) {
            /**
             * @description 注销
             */
            async function logoutFun() {
                await logout();
                removeToken();
                window.localStorage.clear();
                commit('setRoles', []);
                commit('setPermissions', []);

                // 跳转路由
                // router.push({ name: 'login' });
                window.location.href = '/nh-desktop/#/login';
            }
            // 判断是否需要确认
            if (confirm) {
                MessageBox.confirm('确定要注销当前用户吗', '注销用户', {
                        closeOnClickModal: false,
                        closeOnPressEscape: false,
                        type: 'warning'
                    })
                    .then(() => {
                        logoutFun();
                    })
                    .catch(() => {});
            } else {
                logoutFun();
            }
        },

        // 设置用户登录信息
        async setUserInfo({ state }, info) {
            // store 赋值
            state.userInfo = info;
            // 持久化
            window.localStorage.setItem('userInfo', JSON.stringify(info));
        },
        // 读取用户登录信息
        async loadUserInfo({ state }) {
            const userInfo = window.localStorage.getItem('userInfo');
            // store 赋值
            if (userInfo === null) {
                state.userInfo = null;
            } else {
                state.userInfo = JSON.parse(userInfo);
            }
        }
    }
};

export default user;