import axios from 'axios';
import { MessageBox, Message, Loading } from 'element-ui';
import store from '@/store';
import { getToken } from '@/utils/auth';
import errorCode from '@/utils/errorCode';
import { tansParams, blobValidate } from '@/utils/ruoyi';
import cache from '@/plugins/cache';
import { saveAs } from 'file-saver';
import configs from '@/config/';
import { getDataType } from '@/utils/utils';
import sha256 from 'crypto-js/sha256';
let downloadLoadingInstance;
// 是否显示重新登录
export const isRelogin = { show: false };

axios.defaults.headers['Content-Type'] = 'application/json;charset=utf-8';
// 创建axios实例
const service = axios.create({
    // axios中请求配置有baseURL选项，表示请求URL公共部分
    baseURL: configs.baseURL, // process.env.VUE_APP_BASE_API,
    // 超时
    timeout: 1000 * 60 * 2
});

// request拦截器
service.interceptors.request.use(
    (config) => {
        // 是否需要设置 token
        const isToken = (config.headers || {}).isToken === false;
        // 是否需要防止数据重复提交
        const isRepeatSubmit = (config.headers || {}).repeatSubmit === false;

        function randomString(length, chars) {
            var result = '';
            for (var i = length; i > 0; --i) result += chars[Math.floor(Math.random() * chars.length)];
            return result;
        }
        var rString = randomString(
            32,
            '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        );
        // 请求秒
        const integrityEpochSecond = String(new Date().getTime()).substring(0, 10);
        // 随机字符串名
        const integrityNonceStr = rString;
        // 签名
        // let integritySignature = '';
        const salt = 'mon9zhlbvezjl0hkfxph';

        function sortAscii(obj) {
            const arr = [];
            let num = 0;
            for (const i in obj) {
                arr[num] = i;
                num++;
            }
            const sortArr = arr.sort();
            // let sortObj = {};    //完成排序值
            let str = ''; // 自定义排序字符串
            for (const i in sortArr) {
                str += sortArr[i] + '=' + obj[sortArr[i]] + '&';
                // sortObj[sortArr[i]] = obj[sortArr[i]];
            }
            // 去除两侧字符串
            const char = '&';
            str = str.replace(new RegExp('^\\' + char + '+|\\' + char + '+$', 'g'), '');

            return str;
            // return sortObj;
        }

        let isEncryption = true;
        let dataStr = '';
        if (config.method === 'get') {
            for (const i in config.params) {
                if (getDataType(i) !== 'String') {
                    isEncryption = false;
                }
            }
            dataStr = sortAscii(config.params);
        } else {
            for (const i in config.data) {
                if (getDataType(i) !== 'String') {
                    isEncryption = false;
                }
            }
            dataStr = sortAscii(config.data);
        }
        let dataStrs = '';
        if (isEncryption) {
            dataStrs = `integrity-epochsecond=${integrityEpochSecond}&integrity-noncestr=${integrityNonceStr}&${dataStr}&salt=${salt}`;
        }
        const hashDigest = sha256(dataStrs);
        if (getToken() && !isToken) {
            config.headers.Authorization = 'Bearer ' + getToken(); // 让每个请求携带自定义token 请根据实际情况自行修改
            if (isEncryption) {
                config.headers['integrity-epochsecond'] = integrityEpochSecond;
                config.headers['integrity-noncestr'] = integrityNonceStr;
                config.headers['integrity-signature'] = hashDigest;
            }
        }

        // 自定义参数
        const useBase = config.useBase !== false;
        if (useBase) {
            if (config.method === 'get') {
                config.params = {
                    bankCode:
                        (store.state.user.currSelectBank && store.state.user.currSelectBank.bankCode) || '',
                    bankLevel:
                        (store.state.user.currSelectBank && store.state.user.currSelectBank.bankLevel) || '',
                    upCode:
                        (store.state.user.currSelectBank && store.state.user.currSelectBank.upCode) || '000',
                    yMonth:
                        (store.state.user.currAccountMessage && store.state.user.currAccountMessage.yMonth) || '',
                    month:
                        (store.state.user.currAccountMessage && store.state.user.currAccountMessage.yMonth) || '',
                    accountingPeriod:
                        (store.state.user.currAccountMessage &&
                            store.state.user.currAccountMessage.accountingPeriod) ||
                        '',
                    bookId:
                        (store.state.user.currAccountMessage && store.state.user.currAccountMessage.bookId) ||
                        '',
                    year:
                        (store.state.user.currAccountMessage && store.state.user.currAccountMessage.year) || '',
                    initializeStatus:
                        (store.state.user.currAccountMessage &&
                            store.state.user.currAccountMessage.initializeStatus) ||
                        '',
                    ...config.params
                };
            } else {
                config.data = {
                    bankCode:
                        (store.state.user.currSelectBank && store.state.user.currSelectBank.bankCode) || '',
                    bankLevel:
                        (store.state.user.currSelectBank && store.state.user.currSelectBank.bankLevel) || '',
                    upCode:
                        (store.state.user.currSelectBank && store.state.user.currSelectBank.upCode) || '000',
                    yMonth:
                        (store.state.user.currAccountMessage && store.state.user.currAccountMessage.yMonth) || '',
                    month:
                        (store.state.user.currAccountMessage && store.state.user.currAccountMessage.yMonth) || '',
                    accountingPeriod:
                        (store.state.user.currAccountMessage &&
                            store.state.user.currAccountMessage.accountingPeriod) ||
                        '',
                    bookId:
                        (store.state.user.currAccountMessage && store.state.user.currAccountMessage.bookId) ||
                        '',
                    year:
                        (store.state.user.currAccountMessage && store.state.user.currAccountMessage.year) || '',
                    initializeStatus:
                        (store.state.user.currAccountMessage &&
                            store.state.user.currAccountMessage.initializeStatus) ||
                        '',
                    ...config.data
                };
            }
        }

        // get请求映射params参数
        if (config.method === 'get' && config.params) {
            let url = config.url + '?' + tansParams(config.params);
            url = url.slice(0, -1);
            config.params = {};
            config.url = url;
        }
        if (!isRepeatSubmit && (config.method === 'post' || config.method === 'put')) {
            const requestObj = {
                url: config.url,
                data: typeof config.data === 'object' ? JSON.stringify(config.data) : config.data,
                time: new Date().getTime()
            };
            // const sessionObj = cache.session.getJSON('sessionObj');
            // if (sessionObj === undefined || sessionObj === null || sessionObj === '') {
            //     cache.session.setJSON('sessionObj', requestObj);
            // } else {
            //     const sUrl = sessionObj.url; // 请求地址
            //     const sData = sessionObj.data; // 请求数据
            //     const sTime = sessionObj.time; // 请求时间
            //     const interval = 1000; // 间隔时间(ms)，小于此时间视为重复提交
            //     if (
            //         sData === requestObj.data &&
            //         requestObj.time - sTime < interval &&
            //         sUrl === requestObj.url
            //     ) {
            //         const message = '数据正在处理，请勿重复提交';
            //         console.warn(`[${sUrl}]: ` + message);
            //         return Promise.reject(new Error(message));
            //     } else {
            //         cache.session.setJSON('sessionObj', requestObj);
            //     }
            // }
        }
        return config;
    },
    (error) => {
        Promise.reject(error);
    }
);

// 响应拦截器
service.interceptors.response.use(
    (res) => {
        // 未设置状态码则默认成功状态
        const code = res.data.code || 200;
        // 获取错误信息
        const msg = errorCode[code] || res.data.message || errorCode.default;
        // 二进制数据则直接返回
        if (res.request.responseType === 'blob' || res.request.responseType === 'arraybuffer') {
            return res.data;
        }
        if (code === 401) {
            if (!isRelogin.show) {
                isRelogin.show = true;
                MessageBox.confirm('登录状态已过期，您可以继续留在该页面，或者重新登录', '系统提示', {
                        confirmButtonText: '重新登录',
                        cancelButtonText: '取消',
                        closeOnClickModal: false,
                        closeOnPressEscape: false,
                        type: 'warning'
                    })
                    .then(() => {
                        isRelogin.show = false;
                        store.dispatch('Logout');
                    })
                    .catch(() => {
                        isRelogin.show = false;
                    });
            }
            // eslint-disable-next-line prefer-promise-reject-errors
            return Promise.reject('无效的会话，或者会话已过期，请重新登录。');
        } else if (code === 500) {
            Message.closeAll();
            Message({
                message: msg,
                type: 'warning'
            });
            return res.data;
        } else if (code !== 200) {
            Message.closeAll();
            Message({
                message: msg,
                type: 'warning'
            });
            return res.data;
        } else {
            return res.data;
        }
    },
    (error) => {
        let { message } = error;
        if (message === 'Network Error') {
            message = '后端接口连接异常';
        } else if (message.includes('timeout')) {
            message = '服务异常，请联系管理员';
        } else if (message.includes('Request failed with status code')) {
            message = '系统接口' + message.substr(message.length - 3) + '异常';
        }
        Message.closeAll();
        Message({
            message: message,
            type: 'error',
            duration: 5 * 1000
        });
        return Promise.reject(error);
    }
);

// 通用下载方法
export function download(url, params, filename) {
    downloadLoadingInstance = Loading.service({
        text: '正在下载数据，请稍候',
        spinner: 'el-icon-loading',
        background: 'rgba(0, 0, 0, 0.7)'
    });
    return service
        .post(url, params, {
            transformRequest: [
                (params) => {
                    return tansParams(params);
                }
            ],
            headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
            responseType: 'blob'
        })
        .then(async(data) => {
            const isLogin = await blobValidate(data);
            if (isLogin) {
                const blob = new Blob([data]);
                saveAs(blob, filename);
            } else {
                const resText = await data.text();
                const rspObj = JSON.parse(resText);
                const errMsg = errorCode[rspObj.code] || rspObj.msg || errorCode.default;
                Message.error(errMsg);
            }
            downloadLoadingInstance.close();
        })
        .catch((r) => {
            console.error(r);
            Message.error('下载文件出现错误，请联系管理员！');
            downloadLoadingInstance.close();
        });
}

export const downLoadFiles = (url, params, fileName) => {
    try {
        axios
            .get(url, {
                params: params,
                headers: {
                    Authorization: 'Bearer ' + getToken(),
                    'Access-Control-Allow-Origin': '*'
                },
                responseType: 'blob'
            })
            .then((res) => {
                const blob = new Blob([res.data], {
                    type: res.data.type
                });
                const downloadElement = document.createElement('a');
                const href = window.URL.createObjectURL(blob); // 创建下载链接
                downloadElement.href = href;
                downloadElement.download = decodeURIComponent(fileName); // 解码
                document.body.appendChild(downloadElement);
                downloadElement.click();
                document.body.removeChild(downloadElement);
                window.URL.revokeObjectURL(href); // 释放掉blob对象
            });
    } catch (error) {
        this.msgError('您访问的相关资源有误');
    }
};



export function downLoadFiles2(url, params, filename) {
    downloadLoadingInstance = Loading.service({
        text: '正在下载数据，请稍候',
        spinner: 'el-icon-loading',
        background: 'rgba(0, 0, 0, 0.7)'
    });
    return axios
        .get(url, {
            params: params,
            headers: {
                Authorization: 'Bearer ' + getToken(),
                'Access-Control-Allow-Origin': '*'
            },
            responseType: 'blob'
        })
        .then(async(res) => {
            const blob = new Blob([res.data], {
                type: res.data.type
            });
            const downloadElement = document.createElement('a');
            const href = window.URL.createObjectURL(blob); // 创建下载链接
            downloadElement.href = href;
            downloadElement.download = decodeURIComponent(filename); // 解码
            document.body.appendChild(downloadElement);
            downloadElement.click();
            document.body.removeChild(downloadElement);
            window.URL.revokeObjectURL(href); // 释放掉blob对象
            downloadLoadingInstance.close();
        })
        .catch((r) => {
            console.error(r);
            Message.error('下载文件出现错误，请联系管理员！');
            downloadLoadingInstance.close();
        });
}

export default service;
