/**
 * @param {string} path
 * @returns {Boolean}
 */
export function isExternal(path) {
	return /^(https?:|mailto:|tel:)/.test(path);
}

/**
 * @param {string} str
 * @returns {Boolean}
 */
export function validUsername(str) {
	const validMap = ['admin', 'editor'];
	return validMap.indexOf(str.trim()) >= 0;
}

/**
 * @param {string} url
 * @returns {Boolean}
 */
export function validURL(url) {
	const reg =
		/^(https?|ftp):\/\/([a-zA-Z0-9.-]+(:[a-zA-Z0-9.&%$-]+)*@)*((25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9][0-9]?)(\.(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9]?[0-9])){3}|([a-zA-Z0-9-]+\.)*[a-zA-Z0-9-]+\.(com|edu|gov|int|mil|net|org|biz|arpa|info|name|pro|aero|coop|museum|[a-zA-Z]{2}))(:[0-9]+)*(\/($|[a-zA-Z0-9.,?'\\+&%$#=~_-]+))*$/;
	return reg.test(url);
}

/**
 * @param {string} str
 * @returns {Boolean}
 */
export function validLowerCase(str) {
	const reg = /^[a-z]+$/;
	return reg.test(str);
}

/**
 * @param {string} str
 * @returns {Boolean}
 */
export function validUpperCase(str) {
	const reg = /^[A-Z]+$/;
	return reg.test(str);
}

/**
 * @param {string} str
 * @returns {Boolean}
 */
export function validAlphabets(str) {
	const reg = /^[A-Za-z]+$/;
	return reg.test(str);
}

/**
 * @param {string} email
 * @returns {Boolean}
 */
export function validEmail(email) {
	// eslint-disable-next-line no-useless-escape
	const reg =
		// eslint-disable-next-line no-useless-escape
		/^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
	return reg.test(email);
}

/**
 * @param {string} str
 * @returns {Boolean}
 */
export function isString(str) {
	if (typeof str === 'string' || str instanceof String) {
		return true;
	}
	return false;
}

/**
 * @param {Array} arg
 * @returns {Boolean}
 */
export function isArray(arg) {
	if (typeof Array.isArray === 'undefined') {
		return Object.prototype.toString.call(arg) === '[object Array]';
	}
	return Array.isArray(arg);
}

/**
 * 判断是否为身份证号
 * @param {*} data
 */
export function isIdCard(data) {
	var str = data;
	var reg =
		/^[1-9]\d{5}(18|19|20|(3\d))\d{2}((0[1-9])|(1[0-2]))(([0-2][1-9])|10|20|30|31)\d{3}[0-9Xx]$/;
	if (str === '') return false;
	// var len = str.length;
	// if (len === 15) {
	// 	reg = /^(\d{6})()?(\d{2})(\d{2})(\d{2})(\d{3})$/;
	// } else if (len === 18) {
	// 	reg = /^(\d{6})()?(\d{4})(\d{2})(\d{2})(\d{3})(\d|X|x)$/;
	// } else {
	// 	return false;
	// }
	if (!reg.test(str)) return false;
	return true;

	// var part = str.match(reg);
	// var year = len === 15 ? '19' + part[3] : part[3];
	// var date = new Date(year + '/' + part[4] + '/' + part[5]);
	// if (
	// 	date.getFullYear() !== year ||
	// 	date.getMonth() + 1 !== (part[4] | 0) ||
	// 	date.getDate() !== (part[5] | 0)
	// )
	// 	return false;

	// if (len === 15) return true;
	// var wi = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2];
	// var lastcode = '10X98765432';
	// var sum = 0;
	// for (var i = 0; i < 17; i++) {
	// 	sum += parseInt(str.charAt(i)) * wi[i];
	// }
	// return lastcode.charAt(sum % 11) === part[7].toUpperCase();
}

export function isHan(data) {
	var str = data;
	var reg = /^[\u4E00-\u9FA5]+/;
	if (str === '') return false;
	if (!reg.test(str)) return false;
	return true;
}
