'use strict';
const path = require('path');

function resolve(dir) {
	return path.join(__dirname, dir);
}

const CompressionPlugin = require('compression-webpack-plugin');
// 设置不参与构建的库
const externals = {};

const name = '农业平台'; // 网页标题

const port = 2999; // 端口

// vue.config.js 配置说明
// 官方vue.config.js 参考文档 https://cli.vuejs.org/zh/config/#css-loaderoptions
// 这里只列一部分，具体配置参考文档
const systemId = 'nh-sale';
module.exports = {
	transpileDependencies: ['sockjs-client', 'js-base64', 'js-cookie'], // 是否引用第三方 es6+ 的插件进行ie兼容
	publicPath: process.env.NODE_ENV === 'production' ? `/${systemId}` : `/${systemId}`,
	// 在npm run build 或 yarn build 时 ，生成文件的目录名称（要和baseUrl的生产环境路径一致）（默认dist）
	outputDir: systemId,
	// 用于放置生成的静态资源 (js、css、img、fonts) 的；（项目打包之后，静态资源会放在这个文件夹下）
	assetsDir: 'static',
	// 是否开启eslint保存检测，有效值：ture | false | 'error'
	lintOnSave: true, // process.env.NODE_ENV === 'development',
	// 如果你不需要生产环境的 source map，可以将其设置为 false 以加速生产环境构建。
	productionSourceMap: false,
	parallel:false, 
	// webpack-dev-server 相关配置
	devServer: {
		port: port,
		open: true,
	},
	css: {
		loaderOptions: {
			sass: {
				sassOptions: { outputStyle: 'expanded' }
			}
		}
	},
	configureWebpack: {
		externals: process.env.NODE_ENV === 'production' ? externals : {},
		name: name,
		resolve: {
			alias: {
				'@': resolve('src')
			},
			extensions: ['.js', 'css', 'scss', '.vue']
		},
		plugins: [
			// http://doc.ruoyi.vip/ruoyi-vue/other/faq.html#使用gzip解压缩静态文件
			new CompressionPlugin({
				test: /\.(js|css|html)?$/i, // 压缩文件格式
				filename: '[path].gz[query]', // 压缩后的文件名
				algorithm: 'gzip', // 使用gzip压缩
				minRatio: 0.8 // 压缩率小于1才会压缩
			})
		]
	},
	chainWebpack(config) {
		config.plugins.delete('preload'); // TODO: need test
		config.plugins.delete('prefetch'); // TODO: need test
		// config.externals({ './cptable': 'var cptable' });
		config.when(process.env.NODE_ENV !== 'development', (config) => {
			config
				.plugin('ScriptExtHtmlWebpackPlugin')
				.after('html')
				.use('script-ext-html-webpack-plugin', [
					{
						// `runtime` must same as runtimeChunk name. default is `runtime`
						inline: /runtime\..*\.js$/
					}
				])
				.end();
			config.optimization.splitChunks({
				chunks: 'all',
				cacheGroups: {
					libs: {
						name: 'chunk-libs',
						test: /[\\/]node_modules[\\/]/,
						priority: 10,
						chunks: 'initial' // only package third parties that are initially dependent
					},
					elementUI: {
						name: 'chunk-elementUI', // split elementUI into a single package
						priority: 20, // the weight needs to be larger than libs and app or it will be packaged into libs or app
						test: /[\\/]node_modules[\\/]_?element-ui(.*)/ // in order to adapt to cnpm
					},
					commons: {
						name: 'chunk-commons',
						test: resolve('src/components'), // can customize your rules
						minChunks: 3, //  minimum common number
						priority: 5,
						reuseExistingChunk: true
					}
				}
			});
			config.optimization.runtimeChunk('single'),
				{
					from: path.resolve(__dirname, './public/robots.txt'), // 防爬虫文件
					to: './' // 到根目录下
				};
		});

		config.plugin('html').tap((args) => {
			return args;
		});
	}
};
